/* ======================================================================= */
/* CellSimulator.h                                                         */
/*    This file contains the abstraction of Cell Simulator.                */
/*    The Cell Simulator (ViruSIM) is a single system-wide object          */
/*    Abstractions for the environment and comm. are also present here     */
/* ======================================================================= */

#ifndef __CELL_SIM_H__
#define __CELL_SIM_H__

class CellSimulator
{
 private:
  bool growCells ();
  int nelements;
  int nspace;
  bool batch;
  SimCell **cells;
  SimInstructions *instructions;

 public:
  SimCell *getCell (int) const; 
  int numCells () const;
  float minX, maxX, minY, maxY;
  float w_width,w_height;
  float comm_radius;
  int broadcasted;

  bool isBatch () { return batch; } ;

  float rTransX (int x);
  float rTransY (int y);
  int transX (float x);
  int transY (float y);
  CellStateId getState (int i);
  int getX (int i);
  int getY (int i);
  int getR (int i);
  
  void init (bool p_batch, char *p_simFile);
  void showStateSummary (void);
  void writeStateDetails (void);

  SimCell *getNewCell ();
  void freeCell (int index);
  void disperseChemical (int p_source, Emission *);
  
  void interpretSimulationInstructions ();
};

extern void cellOS (int cell_id);
extern void simulator_thread (int dummy);

#endif

