/*
** CellState.cc
*/

# include "basic.h"

CellState::CellState (string p_name, CellTransitionList *p_transitions, EmissionList *p_emissions, int p_color) 
{
  name = p_name;
  transitions = p_transitions;
  emissions = p_emissions;
  color = p_color;
  // cerr << "Color: " << color << "\n";
}

CellState::~CellState ()
{
  delete transitions;
  delete emissions;
}

bool CellState::hasColor () const
{
  return (color != -1);
}

int CellState::getColor () const
{
  assert (hasColor ());
  return color;
}

string CellState::getName () const
{
  return name;
}

string CellState::toString () const 
{
  string s = "state " + name + " { ";

  if (transitions != NULL) {
    s += transitions->toString ();
  }
  
  if (emissions != NULL) {
    s += " emits " + emissions->toString (); 
  }

  s += " }";

  return s;
}

void CellState::resolveStates (CellProgram *cpa)
{
  transitions->resolveStates (cpa);
}

void CellState::resolveChemicals (CellProgram *cpa)
{
  emissions->resolveChemicals (cpa);
  transitions->resolveChemicals (cpa);
}

EmissionList *CellState::getEmissions () const
{
  return emissions;
}

void CellState::getTransition (Environment *env, CellStateId *next1, CellStateId *next2, t_splitKind *mode) 
{
  // for now, ignore environments
  CellTransition *transit = transitions->findMatch (env);

  if (transit == NULL) {
    cerr << "No transition for environment: " << env->toString () << " for state " << toString () << endl;
    *next1 = CELL_NULL_STATE;
    *next2 = CELL_NULL_STATE;
  } else {
    transit->setNextStates (next1, next2, mode);
  }
}





