/*
** CellTransition.cc
*/

# include "basic.h"

CellTransition::CellTransition (ConcentrationList *p_chemicals, string *p_state1)
{
  // cerr << "Creating cell transition to: " << *p_state1 << endl;
  chemicals = p_chemicals;
  nextstate = p_state1;
  nextstateid = -1;
}

string CellTransition::toString ()
{
  if (chemicals != NULL) {
    return chemicals->toString () + " ==> " + *nextstate;
  } else {
    return " ==> " + *nextstate;
  }
}

void CellTransition::resolveStates (CellProgram *cpa)
{
  nextstateid = cpa->lookupState (*nextstate);

  if (nextstateid == -1) {
    cerr << "No matching state: " << *nextstate << endl;
    exit (1);
  }
}

void CellTransition::resolveChemicals (CellProgram *cpa)
{
  if (chemicals != NULL) {
    chemicals->resolveChemicals (cpa);
  }
}

bool CellTransition::matchesEnvironment (Environment *env) const
{
  if (chemicals != NULL) {
    return chemicals->matchesEnvironment (env);
  } else {
    return true;
  }
}

void CellTransition::setNextStates (CellStateId *p_next1, CellStateId *p_next2, t_splitKind *p_mode) const
{
  *p_next1 = nextstateid;
  *p_next2 = CELL_NULL_STATE;
  *p_mode = SPLIT_ERROR;
}

CellSplit::CellSplit (ConcentrationList *p_chemicals, string *p_state1, string *p_state2, t_splitKind p_split)
  : CellTransition (p_chemicals, p_state1)
{
  splitstate = p_state2;
  splitstateid = -1;
  split = p_split;

  // cerr << "Creating cell split: " << toString () << endl;
}

string CellSplit::toString ()
{
  string res;
  if (chemicals != NULL) {
    res = chemicals->toString ();
  }

  res += " ==> [" + *nextstate + " / " + *splitstate + "]_" + (split == SPLIT_NORMAL_1 ? "N" 
                                                               : split == SPLIT_AXIS ? "A"
                                                               : "P");

  return res;
}

void CellSplit::resolveStates (CellProgram *cpa)
{
  CellTransition::resolveStates (cpa);
  splitstateid = cpa->lookupState (*splitstate);

  if (splitstateid == -1) {
    cerr << "No matching state for state in split: " << *splitstate << endl;
    exit (1);
  }
}

void CellSplit::setNextStates (CellStateId *p_next1, CellStateId *p_next2, t_splitKind *p_mode) const
{
  *p_next1 = nextstateid;
  *p_next2 = splitstateid;
  *p_mode = split;

  // cerr << "next states: " << *p_next1 << ", " << *p_next2 << endl;
}








