/*
** CellTransitionList.cc
*/

# include "basic.h"

CellTransitionList::CellTransitionList ()
{
  nelements = 0;
  nspace = BASESIZE; 
  elements = new CellTransition *[nspace];
}

CellTransitionList::~CellTransitionList ()
{
  int i;

  for (i = 0; i < nelements; i++) {
    delete elements[i];
  }

  delete elements;
}

void CellTransitionList::grow ()
{
  int i;
  CellTransition **newelements; 
  int numnew;
  
  if (nelements < BASESIZE)
    {
      numnew = BASESIZE;
    }
  else
    {
      numnew = nelements;
    }

  nspace = numnew + nspace; 

  newelements = new CellTransition *[nspace];
  assert (newelements != NULL);
  
  for (i = 0; i < nelements; i++)
    {
      newelements[i] = elements[i];
    }
  
  delete elements;
  elements = newelements;
}

void CellTransitionList::add (CellTransition *trans)
{
  if (nspace == 0) grow ();
  
  assert (nspace > 0);

  nspace--;
  elements[nelements] = trans;
  nelements++;
}

const string CellTransitionList::toString () const
{
  string res = "";

  for (int i = 0; i < nelements; i++)
    {
      if (i != 0) {
	res += ", ";
      }

      res += elements[i]->toString ();
    }

  return res;
}

void CellTransitionList::resolveStates (CellProgram *cpa)
{
  for (int i = 0; i < nelements; i++)
    {
      elements[i]->resolveStates (cpa);
    }
}

void CellTransitionList::resolveChemicals (CellProgram *cpa)
{
  for (int i = 0; i < nelements; i++)
    {
      elements[i]->resolveChemicals (cpa);
    }
}

CellTransition *CellTransitionList::findMatch (Environment *env) const
{
  if (nelements == 0) {
    return NULL;
  }

  for (int i = nelements - 1; i >= 0; i--) 
    {
      if (elements[i]->matchesEnvironment (env)) {
        return elements[i];
      }
    }

  return NULL;
}
