/*
** ChemicalConcentration.cc
*/

# include "basic.h"
# include <stdio.h>

ChemicalConcentration::ChemicalConcentration (string p_name, float p_lowrange, float p_highrange)
{
  name = p_name;
  lowrange = p_lowrange;
  highrange = p_highrange;
  id = ChemicalId_INVALID;
  cerr << toString () << endl;
}

ChemicalConcentration::ChemicalConcentration (const ChemicalConcentration &old)
{
  name = old.name;
  lowrange = old.lowrange;
  highrange = old.highrange;
  id = old.id;
}

void ChemicalConcentration::resolveChemicals (CellProgram *cpa)
{
  assert (id == ChemicalId_INVALID);
  id = cpa->lookupAddChemical (name);
  cerr << "Resolved: " << this << " ==> " << id << endl;
}

const string ChemicalConcentration::toString () const
{
  string s = "";
  char buf[BUFSIZE];

  s += name;
  s += ": [";
  snprintf (buf, BUFSIZE, "%f", lowrange);
  s += buf;
  s += ", ";
  snprintf (buf, BUFSIZE, "%f", highrange);
  s += buf;
  s += "]";

  return s;
}

bool ChemicalConcentration::matchesEnvironment (Environment *env) const
{
  bool match = false;
  env->startIterator ();

  while (env->hasMore ()) {
    Emission *emit = env->getNext ();

    if (emit->getChemical () == id) {
      float conc = emit->getConcentration ();

      if (conc >= lowrange && conc <= highrange) {
        match = true; 
        break;
      }
    }
  }

  env->finishIterator ();
  return match;
}

