/* ======================================================================= */
/* cell.h                                                                  */
/*    This file contains the abstraction of the Cell in the VirusSIM.      */
/*    A cell contains a VirusTable                                         */
/* ======================================================================= */

#ifndef __CELL_H__
#define __CELL_H__

double normalAngle (double A);
double positiveAngle (double A);
int approximatelySame (double A, double B);

class SimCell
{
private:
  int id;
  
public: // gack! should make all of these private
  MailBox mbox;
  float x, y, z, r;
  double abAxis;
  double abNormal;
  /*int num_neighbours;
    Cell *neighbour_ptr[MAX_NEIGHBOURS];
    double neighbour_distance[MAX_NEIGHBOURS];
    double neighbour_angle[MAX_NEIGHBOURS];*/
  
public:
  // these should be private...
  CellStateId cell_state;
  bool isNewCell;
  CellProgram *cell_program;
  
public:
  SimCell (CellProgram *, int cell_id);
  SimCell (CellProgram *, int cell_id, double px, double py, double pz, double pr, double pAxis, double pNormal);
  void init ();

  bool isAlive () const { return cell_state != CELL_NULL_STATE; }
  void kill () { cell_state = CELL_NULL_STATE; }

  double getDistanceFrom (double p_x, double p_y, double p_z);

  void setValues (double px, double py, double pz, double pr, double pAxis, double pNormal);
  void moveBy (float dx, float dy, float dz);
  void getCoordinates(double &px, double &py, double &pz);
  
  /*void addNeighbour (Cell *ptr, double distance, double angle);
    void removeNeighbour (int index);*/
  void splitUsingNormal_1 (SimCell *daughter);
  void splitUsingNormal_2 (SimCell *daughter);
  void splitUsingAxisPolarised (SimCell *daughter);
  void splitUsingAxisUnpolarised (SimCell *daughter);
  void runCellProgram ();
  int getDisplayColor ();
  string toString ();
  string destructiveToString (); // warning! Modifies by receiving messages
};

#endif

